@@ -40,12 +40,16 @@ extern "C" {
 
 struct isl_basic_map;
 typedef struct isl_basic_map isl_basic_map;
+#ifndef isl_basic_set
 struct isl_basic_set;
 typedef struct isl_basic_set isl_basic_set;
+#endif
 struct isl_map;
 typedef struct isl_map isl_map;
+#ifndef isl_set
 struct isl_set;
 typedef struct isl_set isl_set;
+#endif
 
 unsigned isl_basic_map_n_in(const struct isl_basic_map *bmap);
 unsigned isl_basic_map_n_out(const struct isl_basic_map *bmap);
@@ -88,7 +92,7 @@ int isl_basic_map_is_rational(__isl_keep isl_basic_map *bmap);
 struct isl_basic_map *isl_basic_map_alloc(struct isl_ctx *ctx,
 		unsigned nparam, unsigned in, unsigned out, unsigned extra,
 		unsigned n_eq, unsigned n_ineq);
-__isl_give isl_basic_map *isl_basic_map_identity(__isl_take isl_dim *set_dim);
+__isl_give isl_basic_map *isl_basic_map_identity(__isl_take isl_dim *dim);
 struct isl_basic_map *isl_basic_map_identity_like(struct isl_basic_map *model);
 struct isl_basic_map *isl_basic_map_finalize(struct isl_basic_map *bmap);
 void isl_basic_map_free(__isl_take isl_basic_map *bmap);
@@ -106,6 +110,7 @@ __isl_give isl_basic_map *isl_basic_map_empty(__isl_take isl_dim *dim);
 struct isl_basic_map *isl_basic_map_empty_like(struct isl_basic_map *model);
 struct isl_basic_map *isl_basic_map_empty_like_map(struct isl_map *model);
 __isl_give isl_basic_map *isl_basic_map_universe(__isl_take isl_dim *dim);
+__isl_give isl_basic_map *isl_basic_map_nat_universe(__isl_take isl_dim *dim);
 __isl_give isl_basic_map *isl_basic_map_universe_like(
 		__isl_keep isl_basic_map *bmap);
 __isl_give isl_basic_map *isl_basic_map_remove_redundancies(
@@ -157,8 +162,10 @@ __isl_give isl_map *isl_map_read_from_file(struct isl_ctx *ctx,
 		FILE *input, int nparam);
 __isl_give isl_map *isl_map_read_from_str(isl_ctx *ctx,
 		const char *str, int nparam);
+void isl_basic_map_dump(__isl_keep isl_basic_map *bmap);
 void isl_basic_map_print(__isl_keep isl_basic_map *bmap, FILE *out, int indent,
 	const char *prefix, const char *suffix, unsigned output_format);
+void isl_map_dump(__isl_keep isl_map *map);
 void isl_map_print(__isl_keep isl_map *map, FILE *out, int indent,
 	unsigned output_format);
 __isl_give isl_printer *isl_printer_print_basic_map(
@@ -206,7 +213,8 @@ int isl_basic_map_foreach_lexmin(__isl_keep isl_basic_map *bmap,
 		  void *user),
 	void *user);
 
-void isl_basic_map_dump(__isl_keep isl_basic_map *bmap, FILE *out, int indent);
+void isl_basic_map_print_internal(__isl_keep isl_basic_map *bmap,
+	FILE *out, int indent);
 
 struct isl_basic_map *isl_map_copy_basic_map(struct isl_map *map);
 __isl_give isl_map *isl_map_drop_basic_map(__isl_take isl_map *map,
@@ -228,13 +236,14 @@ struct isl_map *isl_map_alloc(struct isl_ctx *ctx,
 		unsigned nparam, unsigned in, unsigned out, int n,
 		unsigned flags);
 __isl_give isl_map *isl_map_universe(__isl_take isl_dim *dim);
+__isl_give isl_map *isl_map_nat_universe(__isl_take isl_dim *dim);
 __isl_give isl_map *isl_map_empty(__isl_take isl_dim *dim);
 struct isl_map *isl_map_empty_like(struct isl_map *model);
 struct isl_map *isl_map_empty_like_basic_map(struct isl_basic_map *model);
 struct isl_map *isl_map_dup(struct isl_map *map);
 __isl_give isl_map *isl_map_add_basic_map(__isl_take isl_map *map,
 						__isl_take isl_basic_map *bmap);
-__isl_give isl_map *isl_map_identity(__isl_take isl_dim *set_dim);
+__isl_give isl_map *isl_map_identity(__isl_take isl_dim *dim);
 struct isl_map *isl_map_identity_like(struct isl_map *model);
 struct isl_map *isl_map_identity_like_basic_map(struct isl_basic_map *model);
 __isl_give isl_map *isl_map_lex_lt_first(__isl_take isl_dim *dim, unsigned n);
@@ -268,7 +277,14 @@ __isl_give isl_map *isl_map_apply_domain(
 __isl_give isl_map *isl_map_apply_range(
 		__isl_take isl_map *map1,
 		__isl_take isl_map *map2);
-struct isl_map *isl_map_product(struct isl_map *map1, struct isl_map *map2);
+__isl_give isl_map *isl_map_product(__isl_take isl_map *map1,
+	__isl_take isl_map *map2);
+__isl_give isl_basic_map *isl_basic_map_range_product(
+	__isl_take isl_basic_map *bmap1, __isl_take isl_basic_map *bmap2);
+__isl_give isl_map *isl_map_range_product(__isl_take isl_map *map1,
+	__isl_take isl_map *map2);
+__isl_give isl_basic_map *isl_basic_map_flat_product(
+	__isl_take isl_basic_map *bmap1, __isl_take isl_basic_map *bmap2);
 __isl_give isl_map *isl_map_flat_product(__isl_take isl_map *map1,
 	__isl_take isl_map *map2);
 __isl_give isl_map *isl_map_intersect(__isl_take isl_map *map1,
@@ -284,6 +300,9 @@ __isl_give isl_map *isl_map_lower_bound_si(__isl_take isl_map *map,
 		enum isl_dim_type type, unsigned pos, int value);
 __isl_give isl_basic_set *isl_basic_map_deltas(__isl_take isl_basic_map *bmap);
 __isl_give isl_set *isl_map_deltas(__isl_take isl_map *map);
+__isl_give isl_basic_map *isl_basic_map_deltas_map(
+	__isl_take isl_basic_map *bmap);
+__isl_give isl_map *isl_map_deltas_map(__isl_take isl_map *map);
 struct isl_map *isl_map_detect_equalities(struct isl_map *map);
 __isl_give isl_basic_map *isl_map_affine_hull(__isl_take isl_map *map);
 __isl_give isl_basic_map *isl_map_convex_hull(__isl_take isl_map *map);
@@ -321,7 +340,9 @@ __isl_give isl_basic_set *isl_basic_map_wrap(__isl_take isl_basic_map *bmap);
 __isl_give isl_set *isl_map_wrap(__isl_take isl_map *map);
 __isl_give isl_basic_map *isl_basic_set_unwrap(__isl_take isl_basic_set *bset);
 __isl_give isl_map *isl_set_unwrap(__isl_take isl_set *set);
+__isl_give isl_basic_map *isl_basic_map_flatten(__isl_take isl_basic_map *bmap);
 __isl_give isl_map *isl_map_flatten(__isl_take isl_map *map);
+__isl_give isl_basic_set *isl_basic_set_flatten(__isl_take isl_basic_set *bset);
 __isl_give isl_set *isl_set_flatten(__isl_take isl_set *set);
 __isl_give isl_map *isl_set_flatten_map(__isl_take isl_set *set);
 __isl_give isl_set *isl_map_domain(__isl_take isl_map *bmap);
@@ -335,6 +356,8 @@ __isl_give isl_basic_map *isl_basic_map_from_domain(
 __isl_give isl_basic_map *isl_basic_map_from_range(
 	__isl_take isl_basic_set *bset);
 struct isl_map *isl_map_from_range(struct isl_set *set);
+__isl_give isl_basic_map *isl_basic_map_from_domain_and_range(
+	__isl_take isl_basic_set *domain, __isl_take isl_basic_set *range);
 __isl_give isl_map *isl_map_from_domain_and_range(__isl_take isl_set *domain,
 	__isl_take isl_set *range);
 struct isl_map *isl_map_from_set(struct isl_set *set, struct isl_dim *dim);
@@ -352,12 +375,17 @@ int isl_map_is_bijective(__isl_keep isl_map *map);
 int isl_map_is_translation(__isl_keep isl_map *map);
 int isl_map_has_equal_dim(__isl_keep isl_map *map1, __isl_keep isl_map *map2);
 
+int isl_basic_map_can_zip(__isl_keep isl_basic_map *bmap);
+int isl_map_can_zip(__isl_keep isl_map *map);
+__isl_give isl_basic_map *isl_basic_map_zip(__isl_take isl_basic_map *bmap);
+__isl_give isl_map *isl_map_zip(__isl_take isl_map *map);
+
 __isl_give isl_map *isl_map_make_disjoint(__isl_take isl_map *map);
 __isl_give isl_map *isl_basic_map_compute_divs(__isl_take isl_basic_map *bmap);
 __isl_give isl_map *isl_map_compute_divs(__isl_take isl_map *map);
 __isl_give isl_map *isl_map_align_divs(__isl_take isl_map *map);
 
-void isl_map_dump(__isl_keep isl_map *map, FILE *out, int indent);
+void isl_map_print_internal(__isl_keep isl_map *map, FILE *out, int indent);
 
 int isl_map_fast_input_is_fixed(struct isl_map *map,
 		unsigned in, isl_int *val);
@@ -382,8 +410,7 @@ int isl_map_foreach_basic_map(__isl_keep isl_map *map,
 
 __isl_give isl_map *isl_set_lifting(__isl_take isl_set *set);
 
-__isl_give isl_map *isl_map_power(__isl_take isl_map *map, unsigned param,
-	int *exact);
+__isl_give isl_map *isl_map_power(__isl_take isl_map *map, int *exact);
 __isl_give isl_map *isl_map_reaching_path_lengths(__isl_take isl_map *map,
 	int *exact);
 __isl_give isl_map *isl_map_transitive_closure(__isl_take isl_map *map,
