@@ -40,7 +40,7 @@ struct isl_options {
 	unsigned		gbr_only_first;
 
 	#define			ISL_CLOSURE_ISL		0
-	#define			ISL_CLOSURE_OMEGA	1
+	#define			ISL_CLOSURE_BOX		1
 	unsigned		closure;
 
 	#define			ISL_BOUND_BERNSTEIN	0
@@ -54,6 +54,10 @@ struct isl_options {
 	int			bernstein_triangulate;
 
 	int			pip_symmetry;
+
+	#define			ISL_CONVEX_HULL_WRAP	0
+	#define			ISL_CONVEX_HULL_FM	1
+	int			convex;
 };
 
 ISL_ARG_DECL(isl_options, struct isl_options, isl_options_arg)
