@@ -7,9 +7,10 @@
  * Computerwetenschappen, Celestijnenlaan 200A, B-3001 Leuven, Belgium
  */
 
+#include <isl_ctx_private.h>
+#include <isl_map_private.h>
 #include <isl/lp.h>
 #include <isl/map.h>
-#include "isl_map_private.h"
 #include <isl_mat_private.h>
 #include <isl/set.h>
 #include <isl/seq.h>
@@ -174,21 +175,28 @@ error:
 	return -1;
 }
 
-struct isl_basic_set *isl_basic_set_set_rational(struct isl_basic_set *bset)
+__isl_give isl_basic_map *isl_basic_map_set_rational(
+	__isl_take isl_basic_set *bmap)
 {
-	if (!bset)
+	if (!bmap)
 		return NULL;
 
-	if (ISL_F_ISSET(bset, ISL_BASIC_MAP_RATIONAL))
-		return bset;
+	if (ISL_F_ISSET(bmap, ISL_BASIC_MAP_RATIONAL))
+		return bmap;
 
-	bset = isl_basic_set_cow(bset);
-	if (!bset)
+	bmap = isl_basic_map_cow(bmap);
+	if (!bmap)
 		return NULL;
 
-	ISL_F_SET(bset, ISL_BASIC_MAP_RATIONAL);
+	ISL_F_SET(bmap, ISL_BASIC_MAP_RATIONAL);
 
-	return isl_basic_set_finalize(bset);
+	return isl_basic_map_finalize(bmap);
+}
+
+__isl_give isl_basic_set *isl_basic_set_set_rational(
+	__isl_take isl_basic_set *bset)
+{
+	return isl_basic_map_set_rational(bset);
 }
 
 static struct isl_set *isl_set_set_rational(struct isl_set *set)
@@ -290,6 +298,7 @@ static struct isl_basic_set *wrap_constraints(struct isl_set *set)
 		n_ineq += set->p[i]->n_ineq;
 	}
 	lp = isl_basic_set_alloc(set->ctx, 0, dim * set->n, 0, n_eq, n_ineq);
+	lp = isl_basic_set_set_rational(lp);
 	if (!lp)
 		return NULL;
 	lp_dim = isl_basic_set_n_dim(lp);
@@ -1352,6 +1361,9 @@ static struct isl_basic_set *convex_hull_pair(struct isl_basic_set *bset1,
 	isl_basic_set *lin, *aff;
 	int bounded1, bounded2;
 
+	if (bset1->ctx->opt->convex == ISL_CONVEX_HULL_FM)
+		return convex_hull_pair_elim(bset1, bset2);
+
 	aff = isl_set_affine_hull(isl_basic_set_union(isl_basic_set_copy(bset1),
 						    isl_basic_set_copy(bset2)));
 	if (!aff)
@@ -1807,7 +1819,8 @@ static struct isl_basic_set *uset_convex_hull(struct isl_set *set)
 	if (isl_set_n_dim(set) == 1)
 		return convex_hull_1d(set);
 
-	if (isl_set_is_bounded(set))
+	if (isl_set_is_bounded(set) &&
+	    set->ctx->opt->convex == ISL_CONVEX_HULL_WRAP)
 		return uset_convex_hull_wrap(set);
 
 	lin = uset_combined_lineality_space(isl_set_copy(set));
