@@ -576,6 +576,8 @@ __isl_give isl_dim *isl_dim_move(__isl_take isl_dim *dim,
 	enum isl_dim_type dst_type, unsigned dst_pos,
 	enum isl_dim_type src_type, unsigned src_pos, unsigned n)
 {
+	int i;
+
 	if (!dim)
 		return NULL;
 	if (n == 0)
@@ -647,6 +649,18 @@ __isl_give isl_dim *isl_dim_move(__isl_take isl_dim *dim,
 	case isl_dim_out:	dim->n_out -= n; break;
 	}
 
+	if (dst_type != isl_dim_param && src_type != isl_dim_param)
+		return dim;
+
+	for (i = 0; i < 2; ++i) {
+		if (!dim->nested[i])
+			continue;
+		dim->nested[i] = isl_dim_replace(dim->nested[i],
+						 isl_dim_param, dim);
+		if (!dim->nested[i])
+			goto error;
+	}
+
 	return dim;
 error:
 	isl_dim_free(dim);
@@ -722,7 +736,34 @@ error:
 	return NULL;
 }
 
-struct isl_dim *isl_dim_map(struct isl_dim *dim)
+__isl_give isl_dim *isl_dim_range_product(__isl_take isl_dim *left,
+	__isl_take isl_dim *right)
+{
+	isl_dim *dom, *ran1, *ran2, *nest;
+
+	if (!left || !right)
+		goto error;
+
+	isl_assert(left->ctx, match(left, isl_dim_param, right, isl_dim_param),
+			goto error);
+	if (!isl_dim_match(left, isl_dim_in, right, isl_dim_in))
+		isl_die(left->ctx, isl_error_invalid,
+			"domains need to match", goto error);
+
+	dom = isl_dim_domain(isl_dim_copy(left));
+
+	ran1 = isl_dim_range(left);
+	ran2 = isl_dim_range(right);
+	nest = isl_dim_wrap(isl_dim_join(isl_dim_reverse(ran1), ran2));
+
+	return isl_dim_join(isl_dim_reverse(dom), nest);
+error:
+	isl_dim_free(left);
+	isl_dim_free(right);
+	return NULL;
+}
+
+__isl_give isl_dim *isl_dim_map_from_set(__isl_take isl_dim *dim)
 {
 	struct isl_name **names = NULL;
 
@@ -1148,3 +1189,63 @@ error:
 	isl_dim_free(dst);
 	return NULL;
 }
+
+/* Given a dimension specification "dim" of a set, create a dimension
+ * specification for the lift of the set.  In particular, the result
+ * is of the form [dim -> local[..]], with n_local variables in the
+ * range of the wrapped map.
+ */
+__isl_give isl_dim *isl_dim_lift(__isl_take isl_dim *dim, unsigned n_local)
+{
+	isl_dim *local_dim;
+
+	if (!dim)
+		return NULL;
+
+	local_dim = isl_dim_dup(dim);
+	local_dim = isl_dim_drop(local_dim, isl_dim_set, 0, dim->n_out);
+	local_dim = isl_dim_add(local_dim, isl_dim_set, n_local);
+	local_dim = isl_dim_set_tuple_name(local_dim, isl_dim_set, "local");
+	dim = isl_dim_join(isl_dim_from_domain(dim),
+			    isl_dim_from_range(local_dim));
+	dim = isl_dim_wrap(dim);
+	dim = isl_dim_set_tuple_name(dim, isl_dim_set, "lifted");
+
+	return dim;
+}
+
+int isl_dim_can_zip(__isl_keep isl_dim *dim)
+{
+	if (!dim)
+		return -1;
+
+	return dim->nested[0] && dim->nested[1];
+}
+
+__isl_give isl_dim *isl_dim_zip(__isl_take isl_dim *dim)
+{
+	isl_dim *dom, *ran;
+	isl_dim *dom_dom, *dom_ran, *ran_dom, *ran_ran;
+
+	if (!isl_dim_can_zip(dim))
+		isl_die(dim->ctx, isl_error_invalid, "dim cannot be zipped",
+			goto error);
+
+	if (!dim)
+		return 0;
+	dom = isl_dim_unwrap(isl_dim_domain(isl_dim_copy(dim)));
+	ran = isl_dim_unwrap(isl_dim_range(dim));
+	dom_dom = isl_dim_domain(isl_dim_copy(dom));
+	dom_ran = isl_dim_range(dom);
+	ran_dom = isl_dim_domain(isl_dim_copy(ran));
+	ran_ran = isl_dim_range(ran);
+	dom = isl_dim_join(isl_dim_from_domain(dom_dom),
+			   isl_dim_from_range(ran_dom));
+	ran = isl_dim_join(isl_dim_from_domain(dom_ran),
+			   isl_dim_from_range(ran_ran));
+	return isl_dim_join(isl_dim_from_domain(isl_dim_wrap(dom)),
+			    isl_dim_from_range(isl_dim_wrap(ran)));
+error:
+	isl_dim_free(dim);
+	return NULL;
+}
