@@ -14,11 +14,12 @@
 #include <stdio.h>
 #include <string.h>
 #include <strings.h>
+#include <isl_ctx_private.h>
+#include <isl_map_private.h>
 #include <isl/set.h>
 #include <isl/seq.h>
 #include <isl/div.h>
 #include <isl/stream.h>
-#include "isl_map_private.h"
 #include <isl/obj.h>
 #include "isl_polynomial_private.h"
 #include <isl/union_map.h>
@@ -28,6 +29,9 @@ struct variable {
 	char    	    	*name;
 	int	     		 pos;
 	isl_vec			*def;
+	/* non-zero if variable represents a min (-1) or a max (1) */
+	int			 sign;
+	isl_mat			*list;
 	struct variable		*next;
 };
 
@@ -53,6 +57,7 @@ static void variable_free(struct variable *var)
 {
 	while (var) {
 		struct variable *next = var->next;
+		isl_mat_free(var->list);
 		isl_vec_free(var->def);
 		free(var->name);
 		free(var);
@@ -80,6 +85,7 @@ static void vars_drop(struct vars *v, int n)
 	var = v->v;
 	while (--n >= 0) {
 		struct variable *next = var->next;
+		isl_mat_free(var->list);
 		isl_vec_free(var->def);
 		free(var->name);
 		free(var);
@@ -92,7 +98,7 @@ static struct variable *variable_new(struct vars *v, const char *name, int len,
 				int pos)
 {
 	struct variable *var;
-	var = isl_alloc_type(v->ctx, struct variable);
+	var = isl_calloc_type(v->ctx, struct variable);
 	if (!var)
 		goto error;
 	var->name = strdup(name);
@@ -160,11 +166,40 @@ static __isl_give isl_basic_map *set_name(__isl_take isl_basic_map *bmap,
 	return bmap;
 }
 
+/* Obtain next token, with some preprocessing.
+ * In particular, evaluate expressions of the form x^y,
+ * with x and y values.
+ */
+static struct isl_token *next_token(struct isl_stream *s)
+{
+	struct isl_token *tok, *tok2;
+
+	tok = isl_stream_next_token(s);
+	if (!tok || tok->type != ISL_TOKEN_VALUE)
+		return tok;
+	if (!isl_stream_eat_if_available(s, '^'))
+		return tok;
+	tok2 = isl_stream_next_token(s);
+	if (!tok2 || tok2->type != ISL_TOKEN_VALUE) {
+		isl_stream_error(s, tok2, "expecting constant value");
+		goto error;
+	}
+
+	isl_int_pow_ui(tok->u.v, tok->u.v, isl_int_get_ui(tok2->u.v));
+
+	isl_token_free(tok2);
+	return tok;
+error:
+	isl_token_free(tok);
+	isl_token_free(tok2);
+	return NULL;
+}
+
 static int accept_cst_factor(struct isl_stream *s, isl_int *f)
 {
 	struct isl_token *tok;
 
-	tok = isl_stream_next_token(s);
+	tok = next_token(s);
 	if (!tok || tok->type != ISL_TOKEN_VALUE) {
 		isl_stream_error(s, tok, "expecting constant value");
 		goto error;
@@ -183,8 +218,57 @@ error:
 	return -1;
 }
 
+/* Given an affine expression aff, return an affine expression
+ * for aff % d, with d the next token on the stream, which is
+ * assumed to be a constant.
+ *
+ * We introduce an integer division q = [aff/d] and the result
+ * is set to aff - d q.
+ */
+static __isl_give isl_vec *affine_mod(struct isl_stream *s,
+	struct vars *v, __isl_take isl_vec *aff)
+{
+	struct isl_token *tok;
+	struct variable *var;
+	isl_vec *mod;
+
+	tok = next_token(s);
+	if (!tok || tok->type != ISL_TOKEN_VALUE) {
+		isl_stream_error(s, tok, "expecting constant value");
+		goto error;
+	}
+
+	if (vars_add_anon(v) < 0)
+		goto error;
+
+	var = v->v;
+
+	var->def = isl_vec_alloc(s->ctx, 2 + v->n);
+	if (!var->def)
+		goto error;
+	isl_seq_cpy(var->def->el + 1, aff->el, aff->size);
+	isl_int_set_si(var->def->el[1 + aff->size], 0);
+	isl_int_set(var->def->el[0], tok->u.v);
+
+	mod = isl_vec_alloc(v->ctx, 1 + v->n);
+	if (!mod)
+		goto error;
+
+	isl_seq_cpy(mod->el, aff->el, aff->size);
+	isl_int_neg(mod->el[aff->size], tok->u.v);
+
+	isl_vec_free(aff);
+	isl_token_free(tok);
+	return mod;
+error:
+	isl_vec_free(aff);
+	isl_token_free(tok);
+	return NULL;
+}
+
 static struct isl_vec *accept_affine(struct isl_stream *s, struct vars *v);
 static int read_div_definition(struct isl_stream *s, struct vars *v);
+static int read_minmax_definition(struct isl_stream *s, struct vars *v);
 
 static __isl_give isl_vec *accept_affine_factor(struct isl_stream *s,
 	struct vars *v)
@@ -192,7 +276,7 @@ static __isl_give isl_vec *accept_affine_factor(struct isl_stream *s,
 	struct isl_token *tok = NULL;
 	isl_vec *aff = NULL;
 
-	tok = isl_stream_next_token(s);
+	tok = next_token(s);
 	if (!tok) {
 		isl_stream_error(s, NULL, "unexpected EOF");
 		goto error;
@@ -233,7 +317,28 @@ static __isl_give isl_vec *accept_affine_factor(struct isl_stream *s,
 			goto error;
 		if (isl_stream_eat(s, ')'))
 			goto error;
-	} else if (tok->type == '[') {
+	} else if (tok->type == '[' ||
+		    tok->type == ISL_TOKEN_FLOORD ||
+		    tok->type == ISL_TOKEN_CEILD) {
+		int ceil = tok->type == ISL_TOKEN_CEILD;
+		struct variable *var;
+		if (vars_add_anon(v) < 0)
+			goto error;
+		var = v->v;
+		aff = isl_vec_alloc(v->ctx, 1 + v->n);
+		if (!aff)
+			goto error;
+		isl_seq_clr(aff->el, aff->size);
+		isl_int_set_si(aff->el[1 + v->n - 1], ceil ? -1 : 1);
+		isl_stream_push_token(s, tok);
+		tok = NULL;
+		if (read_div_definition(s, v) < 0)
+			goto error;
+		if (ceil)
+			isl_seq_neg(var->def->el + 1, var->def->el + 1,
+				    var->def->size - 1);
+		aff = isl_vec_zero_extend(aff, 1 + v->n);
+	} else if (tok->type == ISL_TOKEN_MIN || tok->type == ISL_TOKEN_MAX) {
 		if (vars_add_anon(v) < 0)
 			goto error;
 		aff = isl_vec_alloc(v->ctx, 1 + v->n);
@@ -243,19 +348,22 @@ static __isl_give isl_vec *accept_affine_factor(struct isl_stream *s,
 		isl_int_set_si(aff->el[1 + v->n - 1], 1);
 		isl_stream_push_token(s, tok);
 		tok = NULL;
-		if (read_div_definition(s, v) < 0)
+		if (read_minmax_definition(s, v) < 0)
 			goto error;
+		aff = isl_vec_zero_extend(aff, 1 + v->n);
 	} else {
 		isl_stream_error(s, tok, "expecting factor");
 		goto error;
 	}
+	if (isl_stream_eat_if_available(s, '%'))
+		return affine_mod(s, v, aff);
 	if (isl_stream_eat_if_available(s, '*')) {
 		isl_int f;
 		isl_int_init(f);
 		isl_int_set_si(f, 1);
 		if (accept_cst_factor(s, &f) < 0) {
 			isl_int_clear(f);
-			goto error;
+			goto error2;
 		}
 		aff = isl_vec_scale(aff, f);
 		isl_int_clear(f);
@@ -264,6 +372,7 @@ static __isl_give isl_vec *accept_affine_factor(struct isl_stream *s,
 	return aff;
 error:
 	isl_token_free(tok);
+error2:
 	isl_vec_free(aff);
 	return NULL;
 }
@@ -280,7 +389,7 @@ static struct isl_vec *accept_affine(struct isl_stream *s, struct vars *v)
 	isl_seq_clr(aff->el, aff->size);
 
 	for (;;) {
-		tok = isl_stream_next_token(s);
+		tok = next_token(s);
 		if (!tok) {
 			isl_stream_error(s, NULL, "unexpected EOF");
 			goto error;
@@ -291,6 +400,9 @@ static struct isl_vec *accept_affine(struct isl_stream *s, struct vars *v)
 			continue;
 		}
 		if (tok->type == '(' || tok->type == '[' ||
+		    tok->type == ISL_TOKEN_MIN || tok->type == ISL_TOKEN_MAX ||
+		    tok->type == ISL_TOKEN_FLOORD ||
+		    tok->type == ISL_TOKEN_CEILD ||
 		    tok->type == ISL_TOKEN_IDENT) {
 			isl_vec *aff2;
 			isl_stream_push_token(s, tok);
@@ -327,7 +439,7 @@ static struct isl_vec *accept_affine(struct isl_stream *s, struct vars *v)
 		}
 		isl_token_free(tok);
 
-		tok = isl_stream_next_token(s);
+		tok = next_token(s);
 		if (tok && tok->type == '-') {
 			sign = -sign;
 			isl_token_free(tok);
@@ -438,7 +550,7 @@ static __isl_give isl_basic_map *read_var_list(struct isl_stream *s,
 	int i = 0;
 	struct isl_token *tok;
 
-	while ((tok = isl_stream_next_token(s)) != NULL) {
+	while ((tok = next_token(s)) != NULL) {
 		int new_name = 0;
 
 		if (tok->type == ISL_TOKEN_IDENT) {
@@ -523,24 +635,51 @@ error:
 	return NULL;
 }
 
+static int read_minmax_definition(struct isl_stream *s, struct vars *v)
+{
+	struct isl_token *tok;
+	struct variable *var;
+
+	var = v->v;
+
+	tok = isl_stream_next_token(s);
+	if (!tok)
+		return -1;
+	var->sign = tok->type == ISL_TOKEN_MIN ? -1 : 1;
+	isl_token_free(tok);
+
+	if (isl_stream_eat(s, '('))
+		return -1;
+
+	var->list = accept_affine_list(s, v);
+	if (!var->list)
+		return -1;
+
+	if (isl_stream_eat(s, ')'))
+		return -1;
+
+	return 0;
+}
+
 static int read_div_definition(struct isl_stream *s, struct vars *v)
 {
 	struct isl_token *tok;
 	int seen_paren = 0;
 	struct isl_vec *aff;
 	struct variable *var;
+	int fc = 0;
 
-	if (isl_stream_eat(s, '['))
-		return -1;
-
-	tok = isl_stream_next_token(s);
-	if (!tok)
-		return -1;
-	if (tok->type == '(') {
-		seen_paren = 1;
-		isl_token_free(tok);
-	} else
-		isl_stream_push_token(s, tok);
+	if (isl_stream_eat_if_available(s, ISL_TOKEN_FLOORD) ||
+	    isl_stream_eat_if_available(s, ISL_TOKEN_CEILD)) {
+		fc = 1;
+		if (isl_stream_eat(s, '('))
+			return -1;
+	} else {
+		if (isl_stream_eat(s, '['))
+			return -1;
+		if (isl_stream_eat_if_available(s, '('))
+			seen_paren = 1;
+	}
 
 	var = v->v;
 
@@ -558,12 +697,17 @@ static int read_div_definition(struct isl_stream *s, struct vars *v)
 
 	isl_vec_free(aff);
 
-	if (seen_paren && isl_stream_eat(s, ')'))
-		return -1;
-	if (isl_stream_eat(s, '/'))
-		return -1;
+	if (fc) {
+		if (isl_stream_eat(s, ','))
+			return -1;
+	} else {
+		if (seen_paren && isl_stream_eat(s, ')'))
+			return -1;
+		if (isl_stream_eat(s, '/'))
+			return -1;
+	}
 
-	tok = isl_stream_next_token(s);
+	tok = next_token(s);
 	if (!tok)
 		return -1;
 	if (tok->type != ISL_TOKEN_VALUE) {
@@ -574,23 +718,28 @@ static int read_div_definition(struct isl_stream *s, struct vars *v)
 	isl_int_set(var->def->el[0], tok->u.v);
 	isl_token_free(tok);
 
-	if (isl_stream_eat(s, ']'))
-		return -1;
+	if (fc) {
+		if (isl_stream_eat(s, ')'))
+			return -1;
+	} else {
+		if (isl_stream_eat(s, ']'))
+			return -1;
+	}
 
 	return 0;
 }
 
 static struct isl_basic_map *add_div_definition(struct isl_stream *s,
-	struct vars *v, struct isl_basic_map *bmap, int k)
+	struct vars *v, struct isl_basic_map *bmap, int pos)
 {
 	struct variable *var = v->v;
+	unsigned o_out = isl_basic_map_offset(bmap, isl_dim_out) - 1;
 
 	if (read_div_definition(s, v) < 0)
 		goto error;
 
-	isl_seq_cpy(bmap->div[k], var->def->el, 2 + v->n);
-
-	if (isl_basic_map_add_div_constraints(bmap, k) < 0)
+	if (isl_basic_map_add_div_constraints_var(bmap, o_out + pos,
+						  var->def->el) < 0)
 		goto error;
 
 	return bmap;
@@ -605,10 +754,9 @@ static struct isl_basic_map *read_defined_var_list(struct isl_stream *s,
 	struct isl_token *tok;
 
 	while ((tok = isl_stream_next_token(s)) != NULL) {
-		int k;
 		int p;
 		int n = v->n;
-		unsigned total = isl_basic_map_total_dim(bmap);
+		unsigned n_out = isl_basic_map_dim(bmap, isl_dim_out);
 
 		if (tok->type != ISL_TOKEN_IDENT)
 			break;
@@ -622,18 +770,15 @@ static struct isl_basic_map *read_defined_var_list(struct isl_stream *s,
 		}
 
 		bmap = isl_basic_map_cow(bmap);
+		bmap = isl_basic_map_add(bmap, isl_dim_out, 1);
 		bmap = isl_basic_map_extend_dim(bmap, isl_dim_copy(bmap->dim),
-						1, 0, 2);
-
-		if ((k = isl_basic_map_alloc_div(bmap)) < 0)
-			goto error;
-		isl_seq_clr(bmap->div[k], 1 + 1 + total);
+						0, 0, 2);
 
 		isl_token_free(tok);
 		tok = isl_stream_next_token(s);
 		if (tok && tok->type == '=') {
 			isl_token_free(tok);
-			bmap = add_div_definition(s, v, bmap, k);
+			bmap = add_div_definition(s, v, bmap, n_out);
 			tok = isl_stream_next_token(s);
 		}
 
@@ -664,7 +809,7 @@ static int next_is_tuple(struct isl_stream *s)
 		isl_stream_push_token(s, tok);
 		return 1;
 	}
-	if (tok->type != ISL_TOKEN_IDENT) {
+	if (tok->type != ISL_TOKEN_IDENT && !tok->is_keyword) {
 		isl_stream_push_token(s, tok);
 		return 0;
 	}
@@ -700,7 +845,7 @@ static __isl_give isl_basic_map *read_tuple(struct isl_stream *s,
 	char *name = NULL;
 
 	tok = isl_stream_next_token(s);
-	if (tok && tok->type == ISL_TOKEN_IDENT) {
+	if (tok && (tok->type == ISL_TOKEN_IDENT || tok->is_keyword)) {
 		name = strdup(tok->u.s);
 		if (!name)
 			goto error;
@@ -756,43 +901,6 @@ error:
 	return NULL;
 }
 
-static struct isl_basic_map *add_constraints(struct isl_stream *s,
-	struct vars *v, struct isl_basic_map *bmap);
-
-static struct isl_basic_map *add_exists(struct isl_stream *s,
-	struct vars *v, struct isl_basic_map *bmap)
-{
-	struct isl_token *tok;
-	int n = v->n;
-	int extra;
-	int seen_paren = 0;
-	int i;
-	unsigned total;
-
-	tok = isl_stream_next_token(s);
-	if (!tok)
-		goto error;
-	if (tok->type == '(') {
-		seen_paren = 1;
-		isl_token_free(tok);
-	} else
-		isl_stream_push_token(s, tok);
-
-	bmap = read_defined_var_list(s, v, bmap);
-	if (!bmap)
-		goto error;
-
-	if (isl_stream_eat(s, ':'))
-		goto error;
-	bmap = add_constraints(s, v, bmap);
-	if (seen_paren && isl_stream_eat(s, ')'))
-		goto error;
-	return bmap;
-error:
-	isl_basic_map_free(bmap);
-	return NULL;
-}
-
 static __isl_give isl_basic_map *construct_constraint(
 	__isl_take isl_basic_map *bmap, enum isl_token_type type,
 	isl_int *left, isl_int *right)
@@ -857,6 +965,42 @@ static int is_comparator(struct isl_token *tok)
 	}
 }
 
+/* Add any variables in the variable list "v" that are not already in "bmap"
+ * as output variables in "bmap".
+ */
+static __isl_give isl_basic_map *add_lifted_divs(__isl_take isl_basic_map *bmap,
+	struct vars *v)
+{
+	int i;
+	int extra;
+	struct variable *var;
+
+	extra = v->n - isl_basic_map_total_dim(bmap);
+
+	if (extra == 0)
+		return bmap;
+
+	bmap = isl_basic_map_add(bmap, isl_dim_out, extra);
+	bmap = isl_basic_map_extend_dim(bmap, isl_basic_map_get_dim(bmap),
+					0, 0, 2 * extra);
+
+	for (i = 0, var = v->v; i < extra; ++i, var = var->next) {
+		if (!var->def)
+			continue;
+		var->def = isl_vec_zero_extend(var->def, 2 + v->n);
+		if (!var->def)
+			goto error;
+		if (isl_basic_map_add_div_constraints_var(bmap, var->pos,
+							  var->def->el) < 0)
+			goto error;
+	}
+
+	return bmap;
+error:
+	isl_basic_map_free(bmap);
+	return NULL;
+}
+
 static struct isl_basic_map *add_constraint(struct isl_stream *s,
 	struct vars *v, struct isl_basic_map *bmap)
 {
@@ -864,16 +1008,6 @@ static struct isl_basic_map *add_constraint(struct isl_stream *s,
 	struct isl_token *tok = NULL;
 	struct isl_mat *aff1 = NULL, *aff2 = NULL;
 
-	tok = isl_stream_next_token(s);
-	if (!tok)
-		goto error;
-	if (tok->type == ISL_TOKEN_EXISTS) {
-		isl_token_free(tok);
-		return add_exists(s, v, bmap);
-	}
-	isl_stream_push_token(s, tok);
-	tok = NULL;
-
 	bmap = isl_basic_map_cow(bmap);
 
 	aff1 = accept_affine_list(s, v);
@@ -895,7 +1029,7 @@ static struct isl_basic_map *add_constraint(struct isl_stream *s,
 		aff1 = isl_mat_add_zero_cols(aff1, aff2->n_col - aff1->n_col);
 		if (!aff1)
 			goto error;
-		bmap = add_divs(bmap, v);
+		bmap = add_lifted_divs(bmap, v);
 		bmap = isl_basic_map_extend_constraints(bmap, 0,
 						aff1->n_row * aff2->n_row);
 		for (i = 0; i < aff1->n_row; ++i)
@@ -925,109 +1059,335 @@ error:
 	return NULL;
 }
 
-static struct isl_basic_map *add_constraints(struct isl_stream *s,
-	struct vars *v, struct isl_basic_map *bmap)
+/* Return first variable, starting at n, representing a min or max,
+ * or NULL if there is no such variable.
+ */
+static struct variable *first_minmax(struct vars *v, int n)
 {
-	struct isl_token *tok;
+	struct variable *first = NULL;
+	struct variable *var;
 
-	for (;;) {
-		bmap = add_constraint(s, v, bmap);
-		if (!bmap)
-			return NULL;
-		tok = isl_stream_next_token(s);
-		if (!tok) {
-			isl_stream_error(s, NULL, "unexpected EOF");
-			goto error;
+	for (var = v->v; var && var->pos >= n; var = var->next)
+		if (var->list)
+			first = var;
+
+	return first;
+}
+
+/* Check whether the variable at the given position only occurs in
+ * inequalities and only with the given sign.
+ */
+static int all_coefficients_of_sign(__isl_keep isl_map *map, int pos, int sign)
+{
+	int i, j;
+
+	if (!map)
+		return -1;
+
+	for (i = 0; i < map->n; ++i) {
+		isl_basic_map *bmap = map->p[i];
+
+		for (j = 0; j < bmap->n_eq; ++j)
+			if (!isl_int_is_zero(bmap->eq[j][1 + pos]))
+				return 0;
+		for (j = 0; j < bmap->n_ineq; ++j) {
+			int s = isl_int_sgn(bmap->ineq[j][1 + pos]);
+			if (s == 0)
+				continue;
+			if (s != sign)
+				return 0;
 		}
-		if (tok->type != ISL_TOKEN_AND)
-			break;
-		isl_token_free(tok);
 	}
-	isl_stream_push_token(s, tok);
 
-	return bmap;
+	return 1;
+}
+
+/* Given a variable m which represents a min or a max of n expressions
+ * b_i, add the constraints
+ *
+ *	m <= b_i
+ *
+ * in case of a min (var->sign < 0) and m >= b_i in case of a max.
+ */
+static __isl_give isl_map *bound_minmax(__isl_take isl_map *map,
+	struct variable *var)
+{
+	int i, k;
+	isl_basic_map *bound;
+	int total;
+
+	total = isl_map_dim(map, isl_dim_all);
+	bound = isl_basic_map_alloc_dim(isl_map_get_dim(map),
+					0, 0, var->list->n_row);
+
+	for (i = 0; i < var->list->n_row; ++i) {
+		k = isl_basic_map_alloc_inequality(bound);
+		if (k < 0)
+			goto error;
+		if (var->sign < 0)
+			isl_seq_cpy(bound->ineq[k], var->list->row[i],
+				    var->list->n_col);
+		else
+			isl_seq_neg(bound->ineq[k], var->list->row[i],
+				    var->list->n_col);
+		isl_int_set_si(bound->ineq[k][1 + var->pos], var->sign);
+		isl_seq_clr(bound->ineq[k] + var->list->n_col,
+			    1 + total - var->list->n_col);
+	}
+
+	map = isl_map_intersect(map, isl_map_from_basic_map(bound));
+
+	return map;
 error:
-	if (tok)
-		isl_token_free(tok);
-	isl_basic_map_free(bmap);
+	isl_basic_map_free(bound);
+	isl_map_free(map);
 	return NULL;
 }
 
-static struct isl_basic_map *read_disjunct(struct isl_stream *s,
-	struct vars *v, __isl_take isl_dim *dim)
+/* Given a variable m which represents a min (or max) of n expressions
+ * b_i, add constraints that assigns the minimal upper bound to m, i.e.,
+ * divide the space into cells where one
+ * of the upper bounds is smaller than all the others and assign
+ * this upper bound to m.
+ *
+ * In particular, if there are n bounds b_i, then the input map
+ * is split into n pieces, each with the extra constraints
+ *
+ *	m = b_i
+ *	b_i <= b_j	for j > i
+ *	b_i <  b_j	for j < i
+ *
+ * in case of a min (var->sign < 0) and similarly in case of a max.
+ *
+ * Note: this function is very similar to set_minimum in isl_tab_pip.c
+ * Perhaps we should try to merge the two.
+ */
+static __isl_give isl_map *set_minmax(__isl_take isl_map *map,
+	struct variable *var)
 {
-	int seen_paren = 0;
-	struct isl_token *tok;
-	struct isl_basic_map *bmap;
+	int i, j, k;
+	isl_basic_map *bmap = NULL;
+	isl_ctx *ctx;
+	isl_map *split = NULL;
+	int total;
+
+	ctx = isl_map_get_ctx(map);
+	total = isl_map_dim(map, isl_dim_all);
+	split = isl_map_alloc_dim(isl_map_get_dim(map),
+				var->list->n_row, ISL_SET_DISJOINT);
+
+	for (i = 0; i < var->list->n_row; ++i) {
+		bmap = isl_basic_map_alloc_dim(isl_map_get_dim(map), 0,
+					       1, var->list->n_row - 1);
+		k = isl_basic_map_alloc_equality(bmap);
+		if (k < 0)
+			goto error;
+		isl_seq_cpy(bmap->eq[k], var->list->row[i], var->list->n_col);
+		isl_int_set_si(bmap->eq[k][1 + var->pos], -1);
+		for (j = 0; j < var->list->n_row; ++j) {
+			if (j == i)
+				continue;
+			k = isl_basic_map_alloc_inequality(bmap);
+			if (k < 0)
+				goto error;
+			if (var->sign < 0)
+				isl_seq_combine(bmap->ineq[k],
+						ctx->one, var->list->row[j],
+						ctx->negone, var->list->row[i],
+						var->list->n_col);
+			else
+				isl_seq_combine(bmap->ineq[k],
+						ctx->negone, var->list->row[j],
+						ctx->one, var->list->row[i],
+						var->list->n_col);
+			isl_seq_clr(bmap->ineq[k] + var->list->n_col,
+				    1 + total - var->list->n_col);
+			if (j < i)
+				isl_int_sub_ui(bmap->ineq[k][0],
+					       bmap->ineq[k][0], 1);
+		}
+		bmap = isl_basic_map_finalize(bmap);
+		split = isl_map_add_basic_map(split, bmap);
+	}
+
+	map = isl_map_intersect(map, split);
+
+	return map;
+error:
+	isl_basic_map_free(bmap);
+	isl_map_free(split);
+	isl_map_free(map);
+	return NULL;
+}
+
+/* Plug in the definitions of all min and max expressions.
+ * If a min expression only appears in inequalities and only
+ * with a positive coefficient, then we can simply bound
+ * the variable representing the min by its defining terms
+ * and similarly for a max expression.
+ * Otherwise, we have to assign the different terms to the
+ * variable under the condition that the assigned term is smaller
+ * than the other terms.
+ */
+static __isl_give isl_map *add_minmax(__isl_take isl_map *map,
+	struct vars *v, int n)
+{
+	int i;
+	struct variable *var;
+
+	while (n < v->n) {
+		var = first_minmax(v, n);
+		if (!var)
+			break;
+		if (all_coefficients_of_sign(map, var->pos, -var->sign))
+			map = bound_minmax(map, var);
+		else
+			map = set_minmax(map, var);
+		n = var->pos + 1;
+	}
+
+	return map;
+}
+
+static isl_map *read_constraint(struct isl_stream *s,
+	struct vars *v, __isl_take isl_basic_map *bmap)
+{
+	int n = v->n;
+	isl_map *map;
 
-	bmap = isl_basic_map_alloc_dim(dim, 0, 0, 0);
 	if (!bmap)
 		return NULL;
 
-	tok = isl_stream_next_token(s);
-	if (!tok)
-		goto error;
-	if (tok->type == '(') {
-		seen_paren = 1;
-		isl_token_free(tok);
-	} else
-		isl_stream_push_token(s, tok);
+	bmap = isl_basic_set_unwrap(isl_basic_set_lift(isl_basic_map_wrap(bmap)));
 
-	bmap = add_constraints(s, v, bmap);
+	bmap = add_constraint(s, v, bmap);
 	bmap = isl_basic_map_simplify(bmap);
 	bmap = isl_basic_map_finalize(bmap);
 
-	if (seen_paren && isl_stream_eat(s, ')'))
-		goto error;
+	map = isl_map_from_basic_map(bmap);
 
-	return bmap;
-error:
-	isl_basic_map_free(bmap);
-	return NULL;
+	map = add_minmax(map, v, n);
+
+	map = isl_set_unwrap(isl_map_domain(map));
+
+	vars_drop(v, v->n - n);
+
+	return map;
 }
 
 static struct isl_map *read_disjuncts(struct isl_stream *s,
-	struct vars *v, __isl_take isl_dim *dim)
+	struct vars *v, __isl_take isl_basic_map *bmap);
+
+static __isl_give isl_map *read_exists(struct isl_stream *s,
+	struct vars *v, __isl_take isl_basic_map *bmap)
 {
-	struct isl_token *tok;
-	struct isl_map *map;
+	int n = v->n;
+	int seen_paren = isl_stream_eat_if_available(s, '(');
+	isl_map *map = NULL;
 
-	tok = isl_stream_next_token(s);
-	if (!tok) {
-		isl_stream_error(s, NULL, "unexpected EOF");
+	bmap = isl_basic_map_from_domain(isl_basic_map_wrap(bmap));
+	bmap = read_defined_var_list(s, v, bmap);
+
+	if (isl_stream_eat(s, ':'))
 		goto error;
-	}
-	if (tok->type == '}') {
-		isl_stream_push_token(s, tok);
-		return isl_map_universe(dim);
-	}
-	isl_stream_push_token(s, tok);
 
-	map = isl_map_empty(isl_dim_copy(dim));
-	for (;;) {
-		struct isl_basic_map *bmap;
-		int n = v->n;
+	map = read_disjuncts(s, v, bmap);
+	map = isl_set_unwrap(isl_map_domain(map));
+	bmap = NULL;
 
-		bmap = read_disjunct(s, v, isl_dim_copy(dim));
-		map = isl_map_union(map, isl_map_from_basic_map(bmap));
+	vars_drop(v, v->n - n);
+	if (seen_paren && isl_stream_eat(s, ')'))
+		goto error;
 
-		vars_drop(v, v->n - n);
-
-		tok = isl_stream_next_token(s);
-		if (!tok || tok->type != ISL_TOKEN_OR)
-			break;
-		isl_token_free(tok);
-	}
-	if (tok)
-		isl_stream_push_token(s, tok);
-
-	isl_dim_free(dim);
 	return map;
 error:
-	isl_dim_free(dim);
+	isl_basic_map_free(bmap);
+	isl_map_free(map);
 	return NULL;
 }
 
+static __isl_give isl_map *read_conjunct(struct isl_stream *s,
+	struct vars *v, __isl_take isl_basic_map *bmap)
+{
+	isl_map *map;
+
+	if (isl_stream_eat_if_available(s, '(')) {
+		map = read_disjuncts(s, v, bmap);
+		if (isl_stream_eat(s, ')'))
+			goto error;
+		return map;
+	}
+
+	if (isl_stream_eat_if_available(s, ISL_TOKEN_EXISTS))
+		return read_exists(s, v, bmap);
+
+	if (isl_stream_eat_if_available(s, ISL_TOKEN_TRUE))
+		return isl_map_from_basic_map(bmap);
+
+	if (isl_stream_eat_if_available(s, ISL_TOKEN_FALSE)) {
+		isl_dim *dim = isl_basic_map_get_dim(bmap);
+		isl_basic_map_free(bmap);
+		return isl_map_empty(dim);
+	}
+		
+	return read_constraint(s, v, bmap);
+error:
+	isl_map_free(map);
+	return NULL;
+}
+
+static __isl_give isl_map *read_conjuncts(struct isl_stream *s,
+	struct vars *v, __isl_take isl_basic_map *bmap)
+{
+	isl_map *map;
+	int negate;
+
+	negate = isl_stream_eat_if_available(s, ISL_TOKEN_NOT);
+	map = read_conjunct(s, v, isl_basic_map_copy(bmap));
+	if (negate) {
+		isl_map *t;
+		t = isl_map_from_basic_map(isl_basic_map_copy(bmap));
+		map = isl_map_subtract(t, map);
+	}
+
+	while (isl_stream_eat_if_available(s, ISL_TOKEN_AND)) {
+		isl_map *map_i;
+
+		negate = isl_stream_eat_if_available(s, ISL_TOKEN_NOT);
+		map_i = read_conjunct(s, v, isl_basic_map_copy(bmap));
+		if (negate)
+			map = isl_map_subtract(map, map_i);
+		else
+			map = isl_map_intersect(map, map_i);
+	}
+
+	isl_basic_map_free(bmap);
+	return map;
+}
+
+static struct isl_map *read_disjuncts(struct isl_stream *s,
+	struct vars *v, __isl_take isl_basic_map *bmap)
+{
+	struct isl_map *map;
+
+	if (isl_stream_next_token_is(s, '}')) {
+		isl_dim *dim = isl_basic_map_get_dim(bmap);
+		isl_basic_map_free(bmap);
+		return isl_map_universe(dim);
+	}
+
+	map = read_conjuncts(s, v, isl_basic_map_copy(bmap));
+	while (isl_stream_eat_if_available(s, ISL_TOKEN_OR)) {
+		isl_map *map_i;
+
+		map_i = read_conjuncts(s, v, isl_basic_map_copy(bmap));
+		map = isl_map_union(map, map_i);
+	}
+
+	isl_basic_map_free(bmap);
+	return map;
+}
+
 static int polylib_pos_to_isl_pos(__isl_keep isl_basic_map *bmap, int pos)
 {
 	if (pos < isl_basic_map_dim(bmap, isl_dim_out))
@@ -1337,7 +1697,7 @@ static __isl_give isl_qpolynomial *read_factor(struct isl_stream *s,
 	struct isl_qpolynomial *qp;
 	struct isl_token *tok;
 
-	tok = isl_stream_next_token(s);
+	tok = next_token(s);
 	if (!tok) {
 		isl_stream_error(s, NULL, "unexpected EOF");
 		return NULL;
@@ -1358,7 +1718,7 @@ static __isl_give isl_qpolynomial *read_factor(struct isl_stream *s,
 		tok2 = isl_stream_next_token(s);
 		if (tok2 && tok2->type == '/') {
 			isl_token_free(tok2);
-			tok2 = isl_stream_next_token(s);
+			tok2 = next_token(s);
 			if (!tok2 || tok2->type != ISL_TOKEN_VALUE) {
 				isl_stream_error(s, tok2, "expected denominator");
 				isl_token_free(tok);
@@ -1389,6 +1749,7 @@ static __isl_give isl_qpolynomial *read_factor(struct isl_stream *s,
 			return NULL;
 		}
 		if (pos >= n) {
+			vars_drop(v, v->n - n);
 			isl_stream_error(s, tok, "unknown identifier");
 			isl_token_free(tok);
 			return NULL;
@@ -1441,7 +1802,7 @@ static __isl_give isl_qpolynomial *read_term(struct isl_stream *s,
 	qp = read_factor(s, bmap, v);
 
 	for (;;) {
-		tok = isl_stream_next_token(s);
+		tok = next_token(s);
 		if (!tok)
 			return qp;
 
@@ -1485,10 +1846,11 @@ static __isl_give isl_map *read_optional_disjuncts(struct isl_stream *s,
 		goto error;
 	}
 	map = isl_map_from_basic_map(isl_basic_map_copy(bmap));
-	if (tok->type == ':') {
+	if (tok->type == ':' ||
+	    (tok->type == ISL_TOKEN_OR && !strcmp(tok->u.s, "|"))) {
 		isl_token_free(tok);
 		map = isl_map_intersect(map,
-			    read_disjuncts(s, v, isl_basic_map_get_dim(bmap)));
+			    read_disjuncts(s, v, isl_basic_map_copy(bmap)));
 	} else
 		isl_stream_push_token(s, tok);
 
@@ -1566,6 +1928,24 @@ error:
 	return obj;
 }
 
+static int is_rational(struct isl_stream *s)
+{
+	struct isl_token *tok;
+
+	tok = isl_stream_next_token(s);
+	if (!tok)
+		return 0;
+	if (tok->type == ISL_TOKEN_RAT && isl_stream_next_token_is(s, ':')) {
+		isl_token_free(tok);
+		isl_stream_eat(s, ':');
+		return 1;
+	}
+
+	isl_stream_push_token(s, tok);
+
+	return 0;
+}
+
 static struct isl_obj obj_read_body(struct isl_stream *s,
 	__isl_take isl_basic_map *bmap, struct vars *v)
 {
@@ -1574,6 +1954,9 @@ static struct isl_obj obj_read_body(struct isl_stream *s,
 	struct isl_obj obj = { isl_obj_set, NULL };
 	int n = v->n;
 
+	if (is_rational(s))
+		bmap = isl_basic_map_set_rational(bmap);
+
 	if (!next_is_tuple(s))
 		return obj_read_poly_or_fold(s, bmap, v, n);
 
@@ -1691,13 +2074,26 @@ static struct isl_obj obj_read(struct isl_stream *s, int nparam)
 	struct vars *v = NULL;
 	struct isl_obj obj = { isl_obj_set, NULL };
 
-	tok = isl_stream_next_token(s);
+	tok = next_token(s);
 	if (!tok) {
 		isl_stream_error(s, NULL, "unexpected EOF");
 		goto error;
 	}
 	if (tok->type == ISL_TOKEN_VALUE) {
+		struct isl_token *tok2;
 		struct isl_map *map;
+
+		tok2 = isl_stream_next_token(s);
+		if (!tok2 || tok2->type != ISL_TOKEN_VALUE ||
+		    isl_int_is_neg(tok2->u.v)) {
+			if (tok2)
+				isl_stream_push_token(s, tok2);
+			obj.type = isl_obj_int;
+			obj.v = isl_int_obj_alloc(s->ctx, tok->u.v);
+			isl_token_free(tok);
+			return obj;
+		}
+		isl_stream_push_token(s, tok2);
 		isl_stream_push_token(s, tok);
 		map = map_read_polylib(s, nparam);
 		if (!map)
@@ -1776,6 +2172,10 @@ static struct isl_obj obj_read(struct isl_stream *s, int nparam)
 		if (!tok || tok->type != ';')
 			break;
 		isl_token_free(tok);
+		if (isl_stream_next_token_is(s, '}')) {
+			tok = isl_stream_next_token(s);
+			break;
+		}
 	}
 
 	if (tok && tok->type == '}') {
@@ -2009,6 +2409,18 @@ error:
 	return NULL;
 }
 
+__isl_give isl_union_map *isl_union_map_read_from_file(isl_ctx *ctx,
+	FILE *input)
+{
+	isl_union_map *umap;
+	struct isl_stream *s = isl_stream_new_file(ctx, input);
+	if (!s)
+		return NULL;
+	umap = isl_stream_read_union_map(s);
+	isl_stream_free(s);
+	return umap;
+}
+
 __isl_give isl_union_map *isl_union_map_read_from_str(struct isl_ctx *ctx,
 		const char *str)
 {
@@ -2021,6 +2433,18 @@ __isl_give isl_union_map *isl_union_map_read_from_str(struct isl_ctx *ctx,
 	return umap;
 }
 
+__isl_give isl_union_set *isl_union_set_read_from_file(isl_ctx *ctx,
+	FILE *input)
+{
+	isl_union_set *uset;
+	struct isl_stream *s = isl_stream_new_file(ctx, input);
+	if (!s)
+		return NULL;
+	uset = isl_stream_read_union_set(s);
+	isl_stream_free(s);
+	return uset;
+}
+
 __isl_give isl_union_set *isl_union_set_read_from_str(struct isl_ctx *ctx,
 		const char *str)
 {
@@ -2068,22 +2492,18 @@ error:
 	return NULL;
 }
 
-static __isl_give isl_vec *vec_read(struct isl_stream *s,
-	unsigned input_format)
+static __isl_give isl_vec *vec_read(struct isl_stream *s)
 {
-	if (input_format == ISL_FORMAT_POLYLIB)
-		return isl_vec_read_polylib(s);
-	isl_assert(s->ctx, 0, return NULL);
+	return isl_vec_read_polylib(s);
 }
 
-struct isl_vec *isl_vec_read_from_file(struct isl_ctx *ctx,
-		FILE *input, unsigned input_format)
+__isl_give isl_vec *isl_vec_read_from_file(isl_ctx *ctx, FILE *input)
 {
 	isl_vec *v;
 	struct isl_stream *s = isl_stream_new_file(ctx, input);
 	if (!s)
 		return NULL;
-	v = vec_read(s, input_format);
+	v = vec_read(s);
 	isl_stream_free(s);
 	return v;
 }
