@@ -8,9 +8,9 @@
  * 91893 Orsay, France 
  */
 
+#include <isl_map_private.h>
 #include <isl_morph.h>
 #include <isl/seq.h>
-#include <isl_map_private.h>
 #include <isl_mat_private.h>
 #include <isl_dim_private.h>
 #include <isl_equalities.h>
@@ -631,6 +631,9 @@ __isl_give isl_basic_set *isl_morph_basic_set(__isl_take isl_morph *morph,
 
 	res = add_strides(res, morph);
 
+	if (isl_basic_set_is_rational(bset))
+		res = isl_basic_set_set_rational(res);
+
 	res = isl_basic_set_simplify(res);
 	res = isl_basic_set_finalize(res);
 
@@ -757,3 +760,19 @@ __isl_give isl_morph *isl_basic_set_full_compression(
 
 	return morph;
 }
+
+__isl_give isl_vec *isl_morph_vec(__isl_take isl_morph *morph,
+	__isl_take isl_vec *vec)
+{
+	if (!morph)
+		goto error;
+
+	vec = isl_mat_vec_product(isl_mat_copy(morph->map), vec);
+
+	isl_morph_free(morph);
+	return vec;
+error:
+	isl_morph_free(morph);
+	isl_vec_free(vec);
+	return NULL;
+}
