@@ -12,12 +12,13 @@
 
 #include <stdlib.h>
 #include <string.h>
+#include <isl_ctx_private.h>
+#include <isl_map_private.h>
 #include <isl/set.h>
 #include <isl/seq.h>
 #include <isl_polynomial_private.h>
 #include <isl_printer_private.h>
 #include <isl_dim_private.h>
-#include <isl_map_private.h>
 #include <isl_mat_private.h>
 #include <isl/union_map.h>
 
@@ -408,9 +409,11 @@ static __isl_give isl_printer *print_nested_map_dim(__isl_take isl_printer *p,
 }
 
 static __isl_give isl_printer *print_dim(__isl_keep isl_dim *dim,
-	__isl_take isl_printer *p, int set, int latex,
+	__isl_take isl_printer *p, int set, int latex, int rational,
 	__isl_keep isl_basic_map *eq)
 {
+	if (rational && !latex)
+		p = isl_printer_print_str(p, "rat: ");
 	if (set)
 		p = print_tuple(dim, p, isl_dim_set, 1, latex, eq);
 	else {
@@ -636,40 +639,19 @@ static __isl_give isl_printer *print_disjunct(__isl_keep isl_basic_map *bmap,
 }
 
 static __isl_give isl_printer *isl_basic_map_print_isl(
-	__isl_keep isl_basic_map *bmap, __isl_take isl_printer *p, int latex)
+	__isl_keep isl_basic_map *bmap, __isl_take isl_printer *p,
+	int set, int latex)
 {
-	int i;
-
-	p = isl_printer_start_line(p);
+	int rational = ISL_F_ISSET(bmap, ISL_BASIC_MAP_RATIONAL);
 	if (isl_basic_map_dim(bmap, isl_dim_param) > 0) {
 		p = print_tuple(bmap->dim, p, isl_dim_param, 0, latex, NULL);
 		p = isl_printer_print_str(p, " -> ");
 	}
 	p = isl_printer_print_str(p, "{ ");
-	p = print_dim(bmap->dim, p, 0, latex, NULL);
+	p = print_dim(bmap->dim, p, set, latex, rational, NULL);
 	p = isl_printer_print_str(p, " : ");
-	p = print_disjunct(bmap, bmap->dim, p, 0, latex);
+	p = print_disjunct(bmap, bmap->dim, p, set, latex);
 	p = isl_printer_print_str(p, " }");
-	p = isl_printer_end_line(p);
-	return p;
-}
-
-static __isl_give isl_printer *isl_basic_set_print_isl(
-	__isl_keep isl_basic_set *bset, __isl_take isl_printer *p, int latex)
-{
-	int i;
-
-	p = isl_printer_start_line(p);
-	if (isl_basic_set_dim(bset, isl_dim_param) > 0) {
-		p = print_tuple(bset->dim, p, isl_dim_param, 0, latex, NULL);
-		p = isl_printer_print_str(p, " -> ");
-	}
-	p = isl_printer_print_str(p, "{ ");
-	p = print_dim(bset->dim, p, 1, latex, NULL);
-	p = isl_printer_print_str(p, " : ");
-	p = print_disjunct((isl_basic_map *)bset, bset->dim, p, 1, latex);
-	p = isl_printer_print_str(p, " }");
-	p = isl_printer_end_line(p);
 	return p;
 }
 
@@ -836,6 +818,7 @@ static __isl_give isl_printer *print_split_map(__isl_take isl_printer *p,
 	struct isl_aff_split *split, int n, int set)
 {
 	int i;
+	int rational;
 
 	for (i = 0; i < n; ++i) {
 		isl_dim *dim;
@@ -843,9 +826,11 @@ static __isl_give isl_printer *print_split_map(__isl_take isl_printer *p,
 		if (!split[i].map)
 			break;
 		dim = split[i].map->dim;
+		rational = split[i].map->n > 0 &&
+		    ISL_F_ISSET(split[i].map->p[0], ISL_BASIC_MAP_RATIONAL);
 		if (i)
 			p = isl_printer_print_str(p, "; ");
-		p = print_dim(dim, p, set, 0, split[i].aff);
+		p = print_dim(dim, p, set, 0, rational, split[i].aff);
 		p = print_disjuncts(split[i].map, p, set, 0);
 	}
 
@@ -856,13 +841,16 @@ static __isl_give isl_printer *isl_map_print_isl_body(__isl_keep isl_map *map,
 	__isl_take isl_printer *p, int set)
 {
 	struct isl_aff_split *split = NULL;
+	int rational;
 
 	if (map->n > 0)
 		split = split_aff(map);
 	if (split) {
 		p = print_split_map(p, split, map->n, set);
 	} else {
-		p = print_dim(map->dim, p, set, 0, NULL);
+		rational = map->n > 0 &&
+		    ISL_F_ISSET(map->p[0], ISL_BASIC_MAP_RATIONAL);
+		p = print_dim(map->dim, p, set, 0, rational, NULL);
 		p = print_disjuncts(map, p, set, 0);
 	}
 	free_split(split, map->n);
@@ -890,7 +878,7 @@ static __isl_give isl_printer *print_latex_map(__isl_keep isl_map *map,
 		p = isl_printer_print_str(p, s_to[1]);
 	}
 	p = isl_printer_print_str(p, s_open_set[1]);
-	p = print_dim(map->dim, p, set, 1, aff);
+	p = print_dim(map->dim, p, set, 1, 0, aff);
 	p = print_disjuncts(map, p, set, 1);
 	p = isl_printer_print_str(p, s_close_set[1]);
 
@@ -927,7 +915,7 @@ __isl_give isl_printer *isl_printer_print_basic_map(__isl_take isl_printer *p,
 	if (!p || !bmap)
 		goto error;
 	if (p->output_format == ISL_FORMAT_ISL)
-		return isl_basic_map_print_isl(bmap, p, 0);
+		return isl_basic_map_print_isl(bmap, p, 0, 0);
 	else if (p->output_format == ISL_FORMAT_OMEGA)
 		return isl_basic_map_print_omega(bmap, p);
 	isl_assert(bmap->ctx, 0, goto error);
@@ -961,7 +949,7 @@ __isl_give isl_printer *isl_printer_print_basic_set(__isl_take isl_printer *p,
 		goto error;
 
 	if (p->output_format == ISL_FORMAT_ISL)
-		return isl_basic_set_print_isl(bset, p, 0);
+		return isl_basic_map_print_isl(bset, p, 1, 0);
 	else if (p->output_format == ISL_FORMAT_POLYLIB)
 		return isl_basic_set_print_polylib(bset, p, 0);
 	else if (p->output_format == ISL_FORMAT_EXT_POLYLIB)
@@ -1168,6 +1156,62 @@ void isl_map_print(__isl_keep isl_map *map, FILE *out, int indent,
 	isl_printer_free(printer);
 }
 
+void isl_basic_map_dump(__isl_keep isl_basic_map *bmap)
+{
+	isl_printer *printer;
+
+	if (!bmap)
+		return;
+
+	printer = isl_printer_to_file(isl_basic_map_get_ctx(bmap), stderr);
+	printer = isl_printer_print_basic_map(printer, bmap);
+	printer = isl_printer_end_line(printer);
+
+	isl_printer_free(printer);
+}
+
+void isl_basic_set_dump(__isl_keep isl_basic_set *bset)
+{
+	isl_printer *printer;
+
+	if (!bset)
+		return;
+
+	printer = isl_printer_to_file(isl_basic_set_get_ctx(bset), stderr);
+	printer = isl_printer_print_basic_set(printer, bset);
+	printer = isl_printer_end_line(printer);
+
+	isl_printer_free(printer);
+}
+
+void isl_map_dump(__isl_keep isl_map *map)
+{
+	isl_printer *printer;
+
+	if (!map)
+		return;
+
+	printer = isl_printer_to_file(isl_map_get_ctx(map), stderr);
+	printer = isl_printer_print_map(printer, map);
+	printer = isl_printer_end_line(printer);
+
+	isl_printer_free(printer);
+}
+
+void isl_set_dump(__isl_keep isl_set *set)
+{
+	isl_printer *printer;
+
+	if (!set)
+		return;
+
+	printer = isl_printer_to_file(isl_set_get_ctx(set), stderr);
+	printer = isl_printer_print_set(printer, set);
+	printer = isl_printer_end_line(printer);
+
+	isl_printer_free(printer);
+}
+
 void isl_union_map_dump(__isl_keep isl_union_map *umap)
 {
 	isl_printer *printer;
@@ -1211,6 +1255,64 @@ void isl_union_pw_qpolynomial_dump(__isl_keep isl_union_pw_qpolynomial *upwqp)
 	isl_printer_free(printer);
 }
 
+void isl_qpolynomial_dump(__isl_keep isl_qpolynomial *qp)
+{
+	isl_printer *printer;
+
+	if (!qp)
+		return;
+
+	printer = isl_printer_to_file(isl_qpolynomial_get_ctx(qp), stderr);
+	printer = isl_printer_print_qpolynomial(printer, qp);
+	printer = isl_printer_end_line(printer);
+
+	isl_printer_free(printer);
+}
+
+void isl_qpolynomial_fold_dump(__isl_keep isl_qpolynomial_fold *fold)
+{
+	isl_printer *printer;
+
+	if (!fold)
+		return;
+
+	printer = isl_printer_to_file(isl_qpolynomial_fold_get_ctx(fold),
+					stderr);
+	printer = isl_printer_print_qpolynomial_fold(printer, fold);
+	printer = isl_printer_end_line(printer);
+
+	isl_printer_free(printer);
+}
+
+void isl_pw_qpolynomial_dump(__isl_keep isl_pw_qpolynomial *pwqp)
+{
+	isl_printer *printer;
+
+	if (!pwqp)
+		return;
+
+	printer = isl_printer_to_file(isl_pw_qpolynomial_get_ctx(pwqp), stderr);
+	printer = isl_printer_print_pw_qpolynomial(printer, pwqp);
+	printer = isl_printer_end_line(printer);
+
+	isl_printer_free(printer);
+}
+
+void isl_pw_qpolynomial_fold_dump(__isl_keep isl_pw_qpolynomial_fold *pwf)
+{
+	isl_printer *printer;
+
+	if (!pwf)
+		return;
+
+	printer = isl_printer_to_file(
+			isl_pw_qpolynomial_fold_get_ctx(pwf), stderr);
+	printer = isl_printer_print_pw_qpolynomial_fold(printer, pwf);
+	printer = isl_printer_end_line(printer);
+
+	isl_printer_free(printer);
+}
+
 void isl_union_pw_qpolynomial_fold_dump(
 	__isl_keep isl_union_pw_qpolynomial_fold *upwf)
 {
@@ -1416,18 +1518,6 @@ static __isl_give isl_printer *qpolynomial_fold_print(
 	return p;
 }
 
-__isl_give isl_printer *isl_printer_print_qpolynomial_fold(
-	__isl_take isl_printer *p, __isl_keep isl_qpolynomial_fold *fold)
-{
-	if  (!p || !fold)
-		goto error;
-	p = qpolynomial_fold_print(fold, p);
-	return p;
-error:
-	isl_printer_free(p);
-	return NULL;
-}
-
 void isl_qpolynomial_fold_print(__isl_keep isl_qpolynomial_fold *fold,
 	FILE *out, unsigned output_format)
 {
@@ -1454,7 +1544,7 @@ static __isl_give isl_printer *isl_pwqp_print_isl_body(
 			p = isl_printer_print_str(p, "; ");
 		if (isl_dim_size(pwqp->dim, isl_dim_set) > 0 ||
 		    isl_dim_is_named_or_nested(pwqp->dim, isl_dim_set)) {
-			p = print_dim(pwqp->p[i].set->dim, p, 1, 0, NULL);
+			p = print_dim(pwqp->p[i].set->dim, p, 1, 0, 0, NULL);
 			p = isl_printer_print_str(p, " -> ");
 		}
 		p = isl_printer_print_qpolynomial(p, pwqp->p[i].qp);
@@ -1478,7 +1568,7 @@ static __isl_give isl_printer *print_pw_qpolynomial_isl(
 	if (pwqp->n == 0) {
 		if (isl_dim_size(pwqp->dim, isl_dim_set) > 0 ||
 		    isl_dim_is_named_or_nested(pwqp->dim, isl_dim_set)) {
-			p = print_dim(pwqp->dim, p, 1, 0, NULL);
+			p = print_dim(pwqp->dim, p, 1, 0, 0, NULL);
 			p = isl_printer_print_str(p, " -> ");
 		}
 		p = isl_printer_print_str(p, "0");
@@ -1516,7 +1606,7 @@ static __isl_give isl_printer *isl_pwf_print_isl_body(
 			p = isl_printer_print_str(p, "; ");
 		if (isl_dim_size(pwf->dim, isl_dim_set) > 0 ||
 		    isl_dim_is_named_or_nested(pwf->dim, isl_dim_set)) {
-			p = print_dim(pwf->p[i].set->dim, p, 1, 0, NULL);
+			p = print_dim(pwf->p[i].set->dim, p, 1, 0, 0, NULL);
 			p = isl_printer_print_str(p, " -> ");
 		}
 		p = qpolynomial_fold_print(pwf->p[i].fold, p);
@@ -1537,7 +1627,7 @@ static __isl_give isl_printer *print_pw_qpolynomial_fold_isl(
 	if (pwf->n == 0) {
 		if (isl_dim_size(pwf->dim, isl_dim_set) > 0 ||
 		    isl_dim_is_named_or_nested(pwf->dim, isl_dim_set)) {
-			p = print_dim(pwf->dim, p, 1, 0, NULL);
+			p = print_dim(pwf->dim, p, 1, 0, 0, NULL);
 			p = isl_printer_print_str(p, " -> ");
 		}
 		p = isl_printer_print_str(p, "0");
@@ -1828,6 +1918,22 @@ static __isl_give isl_printer *print_qpolynomial_fold_c(
 	return p;
 }
 
+__isl_give isl_printer *isl_printer_print_qpolynomial_fold(
+	__isl_take isl_printer *p, __isl_keep isl_qpolynomial_fold *fold)
+{
+	if  (!p || !fold)
+		goto error;
+	if (p->output_format == ISL_FORMAT_ISL)
+		return qpolynomial_fold_print(fold, p);
+	else if (p->output_format == ISL_FORMAT_C)
+		return print_qpolynomial_fold_c(p, fold->dim, fold);
+	isl_die(p->ctx, isl_error_unsupported, "unsupported output format",
+		goto error);
+error:
+	isl_printer_free(p);
+	return NULL;
+}
+
 static __isl_give isl_printer *print_pw_qpolynomial_fold_c(
 	__isl_take isl_printer *p, __isl_keep isl_pw_qpolynomial_fold *pwf)
 {
