@@ -7,6 +7,7 @@
  * Computerwetenschappen, Celestijnenlaan 200A, B-3001 Leuven, Belgium
  */
 
+#include <isl_ctx_private.h>
 #include <isl_mat_private.h>
 #include "isl_map_private.h"
 #include "isl_tab.h"
@@ -1083,6 +1084,11 @@ int isl_tab_pivot(struct isl_tab *tab, int row, int col)
 	struct isl_tab_var *var;
 	unsigned off = 2 + tab->M;
 
+	if (tab->mat->ctx->abort) {
+		isl_ctx_set_error(tab->mat->ctx, isl_error_abort);
+		return -1;
+	}
+
 	isl_int_swap(mat->row[row][0], mat->row[row][off + col]);
 	sgn = isl_int_sgn(mat->row[row][0]);
 	if (sgn < 0) {
@@ -1517,6 +1523,43 @@ int isl_tab_kill_col(struct isl_tab *tab, int col)
 	}
 }
 
+static int row_is_manifestly_non_integral(struct isl_tab *tab, int row)
+{
+	unsigned off = 2 + tab->M;
+
+	if (tab->M && !isl_int_eq(tab->mat->row[row][2],
+				  tab->mat->row[row][0]))
+		return 0;
+	if (isl_seq_first_non_zero(tab->mat->row[row] + off + tab->n_dead,
+				    tab->n_col - tab->n_dead) != -1)
+		return 0;
+
+	return !isl_int_is_divisible_by(tab->mat->row[row][1],
+					tab->mat->row[row][0]);
+}
+
+/* For integer tableaus, check if any of the coordinates are stuck
+ * at a non-integral value.
+ */
+static int tab_is_manifestly_empty(struct isl_tab *tab)
+{
+	int i;
+
+	if (tab->empty)
+		return 1;
+	if (tab->rational)
+		return 0;
+
+	for (i = 0; i < tab->n_var; ++i) {
+		if (!tab->var[i].is_row)
+			continue;
+		if (row_is_manifestly_non_integral(tab, tab->var[i].index))
+			return 1;
+	}
+
+	return 0;
+}
+
 /* Row variable "var" is non-negative and cannot attain any values
  * larger than zero.  This means that the coefficients of the unrestricted
  * column variables are zero and that the coefficients of the non-negative
@@ -1551,6 +1594,8 @@ static int close_row(struct isl_tab *tab, struct isl_tab_var *var)
 	}
 	if (isl_tab_mark_redundant(tab, var->index) < 0)
 		return -1;
+	if (tab_is_manifestly_empty(tab) && isl_tab_mark_empty(tab) < 0)
+		return -1;
 	return 0;
 }
 
@@ -1630,6 +1675,9 @@ int isl_tab_allocate_var(struct isl_tab *tab)
  *		d_r   d_r        d_r d_x/g                m
  *
  *	with g the gcd of d_r and d_x and m the lcm of d_r and d_x.
+ *
+ * If tab->M is set, then, internally, each variable x is represented
+ * as x' - M.  We then also need no subtract k d_r from the coefficient of M.
  */
 int isl_tab_add_row(struct isl_tab *tab, isl_int *line)
 {
@@ -2057,7 +2105,35 @@ error:
 	return -1;
 }
 
-/* Add an extra div, prescrived by "div" to the tableau and
+/* Check whether the div described by "div" is obviously non-negative.
+ * If we are using a big parameter, then we will encode the div
+ * as div' = M + div, which is always non-negative.
+ * Otherwise, we check whether div is a non-negative affine combination
+ * of non-negative variables.
+ */
+static int div_is_nonneg(struct isl_tab *tab, __isl_keep isl_vec *div)
+{
+	int i;
+
+	if (tab->M)
+		return 1;
+
+	if (isl_int_is_neg(div->el[1]))
+		return 0;
+
+	for (i = 0; i < tab->n_var; ++i) {
+		if (isl_int_is_neg(div->el[2 + i]))
+			return 0;
+		if (isl_int_is_zero(div->el[2 + i]))
+			continue;
+		if (!tab->var[i].is_nonneg)
+			return 0;
+	}
+
+	return 1;
+}
+
+/* Add an extra div, prescribed by "div" to the tableau and
  * the associated bmap (which is assumed to be non-NULL).
  *
  * If add_ineq is not NULL, then this function is used instead
@@ -2069,7 +2145,6 @@ error:
 int isl_tab_add_div(struct isl_tab *tab, __isl_keep isl_vec *div,
 	int (*add_ineq)(void *user, isl_int *), void *user)
 {
-	int i;
 	int r;
 	int k;
 	int nonneg;
@@ -2079,15 +2154,7 @@ int isl_tab_add_div(struct isl_tab *tab, __isl_keep isl_vec *div,
 
 	isl_assert(tab->mat->ctx, tab->bmap, return -1);
 
-	for (i = 0; i < tab->n_var; ++i) {
-		if (isl_int_is_neg(div->el[2 + i]))
-			break;
-		if (isl_int_is_zero(div->el[2 + i]))
-			continue;
-		if (!tab->var[i].is_nonneg)
-			break;
-	}
-	nonneg = i == tab->n_var && !isl_int_is_neg(div->el[1]);
+	nonneg = div_is_nonneg(tab, div);
 
 	if (isl_tab_extend_cons(tab, 3) < 0)
 		return -1;
@@ -2709,16 +2776,17 @@ int isl_tab_is_equality(struct isl_tab *tab, int con)
 
 	off = 2 + tab->M;
 	return isl_int_is_zero(tab->mat->row[row][1]) &&
-		isl_seq_first_non_zero(tab->mat->row[row] + 2 + tab->n_dead,
+		(!tab->M || isl_int_is_zero(tab->mat->row[row][2])) &&
+		isl_seq_first_non_zero(tab->mat->row[row] + off + tab->n_dead,
 					tab->n_col - tab->n_dead) == -1;
 }
 
-/* Return the minimial value of the affine expression "f" with denominator
+/* Return the minimal value of the affine expression "f" with denominator
  * "denom" in *opt, *opt_denom, assuming the tableau is not empty and
  * the expression cannot attain arbitrarily small values.
  * If opt_denom is NULL, then *opt is rounded up to the nearest integer.
  * The return value reflects the nature of the result (empty, unbounded,
- * minmimal value returned in *opt).
+ * minimal value returned in *opt).
  */
 enum isl_lp_result isl_tab_min(struct isl_tab *tab,
 	isl_int *f, isl_int denom, isl_int *opt, isl_int *opt_denom,
@@ -2740,8 +2808,6 @@ enum isl_lp_result isl_tab_min(struct isl_tab *tab,
 	if (r < 0)
 		return isl_lp_error;
 	var = &tab->con[r];
-	isl_int_mul(tab->mat->row[var->index][0],
-		    tab->mat->row[var->index][0], denom);
 	for (;;) {
 		int row, col;
 		find_pivot(tab, var, var, -1, &row, &col);
@@ -2754,6 +2820,8 @@ enum isl_lp_result isl_tab_min(struct isl_tab *tab,
 		if (isl_tab_pivot(tab, row, col) < 0)
 			return isl_lp_error;
 	}
+	isl_int_mul(tab->mat->row[var->index][0],
+		    tab->mat->row[var->index][0], denom);
 	if (ISL_FL_ISSET(flags, ISL_TAB_SAVE_DUAL)) {
 		int i;
 
@@ -3050,9 +3118,9 @@ int isl_tab_rollback(struct isl_tab *tab, struct isl_tab_undo *snap)
  * In particular, if the row has been reduced to the constant -1,
  * then we know the inequality is adjacent (but opposite) to
  * an equality in the tableau.
- * If the row has been reduced to r = -1 -r', with r' an inequality
- * of the tableau, then the inequality is adjacent (but opposite)
- * to the inequality r'.
+ * If the row has been reduced to r = c*(-1 -r'), with r' an inequality
+ * of the tableau and c a positive constant, then the inequality
+ * is adjacent (but opposite) to the inequality r'.
  */
 static enum isl_ineq_type separation_type(struct isl_tab *tab, unsigned row)
 {
@@ -3064,15 +3132,18 @@ static enum isl_ineq_type separation_type(struct isl_tab *tab, unsigned row)
 
 	if (!isl_int_is_one(tab->mat->row[row][0]))
 		return isl_ineq_separate;
-	if (!isl_int_is_negone(tab->mat->row[row][1]))
-		return isl_ineq_separate;
 
 	pos = isl_seq_first_non_zero(tab->mat->row[row] + off + tab->n_dead,
 					tab->n_col - tab->n_dead);
-	if (pos == -1)
-		return isl_ineq_adj_eq;
+	if (pos == -1) {
+		if (isl_int_is_negone(tab->mat->row[row][1]))
+			return isl_ineq_adj_eq;
+		else
+			return isl_ineq_separate;
+	}
 
-	if (!isl_int_is_negone(tab->mat->row[row][off + tab->n_dead + pos]))
+	if (!isl_int_eq(tab->mat->row[row][1],
+			tab->mat->row[row][off + tab->n_dead + pos]))
 		return isl_ineq_separate;
 
 	pos = isl_seq_first_non_zero(
@@ -3243,5 +3314,5 @@ void isl_tab_dump(struct isl_tab *tab, FILE *out, int indent)
 	tab->mat->n_row = r;
 	tab->mat->n_col = c;
 	if (tab->bmap)
-		isl_basic_map_dump(tab->bmap, out, indent);
+		isl_basic_map_print_internal(tab->bmap, out, indent);
 }
