@@ -8,12 +8,12 @@
  * 91893 Orsay, France 
  */
 
+#include <isl_map_private.h>
 #include <isl/ctx.h>
 #include <isl/hash.h>
 #include <isl/map.h>
 #include <isl/set.h>
 #include <isl_dim_private.h>
-#include <isl_map_private.h>
 #include <isl_union_map_private.h>
 #include <isl/union_set.h>
 
@@ -877,6 +877,25 @@ __isl_give isl_union_set *isl_union_set_product(__isl_take isl_union_set *uset1,
 	return isl_union_map_product(uset1, uset2);
 }
 
+static int range_product_entry(void **entry, void *user)
+{
+	struct isl_union_map_bin_data *data = user;
+	isl_map *map2 = *entry;
+
+	map2 = isl_map_range_product(isl_map_copy(data->map),
+				     isl_map_copy(map2));
+
+	data->res = isl_union_map_add_map(data->res, map2);
+
+	return 0;
+}
+
+__isl_give isl_union_map *isl_union_map_range_product(
+	__isl_take isl_union_map *umap1, __isl_take isl_union_map *umap2)
+{
+	return bin_op(umap1, umap2, &range_product_entry);
+}
+
 __isl_give isl_union_map *isl_union_map_from_range(
 	__isl_take isl_union_set *uset)
 {
@@ -897,10 +916,9 @@ __isl_give isl_union_map *isl_union_map_from_domain_and_range(
 }
 
 static __isl_give isl_union_map *un_op(__isl_take isl_union_map *umap,
-	int (*fn)(void **, void *), int cow)
+	int (*fn)(void **, void *))
 {
-	if (cow)
-		umap = isl_union_map_cow(umap);
+	umap = isl_union_map_cow(umap);
 	if (!umap)
 		return NULL;
 
@@ -925,7 +943,7 @@ static int affine_entry(void **entry, void *user)
 __isl_give isl_union_map *isl_union_map_affine_hull(
 	__isl_take isl_union_map *umap)
 {
-	return un_op(umap, &affine_entry, 1);
+	return un_op(umap, &affine_entry);
 }
 
 __isl_give isl_union_set *isl_union_set_affine_hull(
@@ -946,7 +964,7 @@ static int polyhedral_entry(void **entry, void *user)
 __isl_give isl_union_map *isl_union_map_polyhedral_hull(
 	__isl_take isl_union_map *umap)
 {
-	return un_op(umap, &polyhedral_entry, 1);
+	return un_op(umap, &polyhedral_entry);
 }
 
 __isl_give isl_union_set *isl_union_set_polyhedral_hull(
@@ -955,19 +973,64 @@ __isl_give isl_union_set *isl_union_set_polyhedral_hull(
 	return isl_union_map_polyhedral_hull(uset);
 }
 
-static int coalesce_entry(void **entry, void *user)
+static int simple_entry(void **entry, void *user)
 {
 	isl_map **map = (isl_map **)entry;
 
-	*map = isl_map_coalesce(*map);
+	*map = isl_map_from_basic_map(isl_map_simple_hull(*map));
 
 	return *map ? 0 : -1;
 }
 
+__isl_give isl_union_map *isl_union_map_simple_hull(
+	__isl_take isl_union_map *umap)
+{
+	return un_op(umap, &simple_entry);
+}
+
+__isl_give isl_union_set *isl_union_set_simple_hull(
+	__isl_take isl_union_set *uset)
+{
+	return isl_union_map_simple_hull(uset);
+}
+
+static int inplace_entry(void **entry, void *user)
+{
+	__isl_give isl_map *(*fn)(__isl_take isl_map *);
+	isl_map **map = (isl_map **)entry;
+	isl_map *copy;
+
+	fn = *(__isl_give isl_map *(**)(__isl_take isl_map *)) user;
+	copy = fn(isl_map_copy(*map));
+	if (!copy)
+		return -1;
+
+	isl_map_free(*map);
+	*map = copy;
+
+	return 0;
+}
+
+static __isl_give isl_union_map *inplace(__isl_take isl_union_map *umap,
+	__isl_give isl_map *(*fn)(__isl_take isl_map *))
+{
+	if (!umap)
+		return NULL;
+
+	if (isl_hash_table_foreach(umap->dim->ctx, &umap->table,
+				    &inplace_entry, &fn) < 0)
+		goto error;
+
+	return umap;
+error:
+	isl_union_map_free(umap);
+	return NULL;
+}
+
 __isl_give isl_union_map *isl_union_map_coalesce(
 	__isl_take isl_union_map *umap)
 {
-	return un_op(umap, &coalesce_entry, 0);
+	return inplace(umap, &isl_map_coalesce);
 }
 
 __isl_give isl_union_set *isl_union_set_coalesce(
@@ -976,19 +1039,22 @@ __isl_give isl_union_set *isl_union_set_coalesce(
 	return isl_union_map_coalesce(uset);
 }
 
-static int compute_divs_entry(void **entry, void *user)
+__isl_give isl_union_map *isl_union_map_detect_equalities(
+	__isl_take isl_union_map *umap)
 {
-	isl_map **map = (isl_map **)entry;
+	return inplace(umap, &isl_map_detect_equalities);
+}
 
-	*map = isl_map_compute_divs(*map);
-
-	return *map ? 0 : -1;
+__isl_give isl_union_set *isl_union_set_detect_equalities(
+	__isl_take isl_union_set *uset)
+{
+	return isl_union_map_detect_equalities(uset);
 }
 
 __isl_give isl_union_map *isl_union_map_compute_divs(
 	__isl_take isl_union_map *umap)
 {
-	return un_op(umap, &compute_divs_entry, 0);
+	return inplace(umap, &isl_map_compute_divs);
 }
 
 __isl_give isl_union_set *isl_union_set_compute_divs(
@@ -1009,7 +1075,7 @@ static int lexmin_entry(void **entry, void *user)
 __isl_give isl_union_map *isl_union_map_lexmin(
 	__isl_take isl_union_map *umap)
 {
-	return un_op(umap, &lexmin_entry, 1);
+	return un_op(umap, &lexmin_entry);
 }
 
 __isl_give isl_union_set *isl_union_set_lexmin(
@@ -1030,7 +1096,7 @@ static int lexmax_entry(void **entry, void *user)
 __isl_give isl_union_map *isl_union_map_lexmax(
 	__isl_take isl_union_map *umap)
 {
-	return un_op(umap, &lexmax_entry, 1);
+	return un_op(umap, &lexmax_entry);
 }
 
 __isl_give isl_union_set *isl_union_set_lexmax(
@@ -1156,6 +1222,26 @@ __isl_give isl_union_set *isl_union_map_deltas(__isl_take isl_union_map *umap)
 	return cond_un_op(umap, &deltas_entry);
 }
 
+static int deltas_map_entry(void **entry, void *user)
+{
+	isl_map *map = *entry;
+	isl_union_map **res = user;
+
+	if (!isl_dim_tuple_match(map->dim, isl_dim_in, map->dim, isl_dim_out))
+		return 0;
+
+	*res = isl_union_map_add_map(*res,
+				     isl_map_deltas_map(isl_map_copy(map)));
+
+	return 0;
+}
+
+__isl_give isl_union_map *isl_union_map_deltas_map(
+	__isl_take isl_union_map *umap)
+{
+	return cond_un_op(umap, &deltas_map_entry);
+}
+
 static int identity_entry(void **entry, void *user)
 {
 	isl_set *set = *entry;
@@ -1333,6 +1419,9 @@ __isl_give isl_basic_map *isl_union_map_sample(__isl_take isl_union_map *umap)
 	    !sample)
 		goto error;
 
+	if (!sample)
+		sample = isl_basic_map_empty(isl_union_map_get_dim(umap));
+
 	isl_union_map_free(umap);
 
 	return sample;
@@ -1377,3 +1466,21 @@ int isl_union_set_is_empty(__isl_keep isl_union_set *uset)
 {
 	return isl_union_map_is_empty(uset);
 }
+
+static int zip_entry(void **entry, void *user)
+{
+	isl_map *map = *entry;
+	isl_union_map **res = user;
+
+	if (!isl_map_can_zip(map))
+		return 0;
+
+	*res = isl_union_map_add_map(*res, isl_map_zip(isl_map_copy(map)));
+
+	return 0;
+}
+
+__isl_give isl_union_map *isl_union_map_zip(__isl_take isl_union_map *umap)
+{
+	return cond_un_op(umap, &zip_entry);
+}
