@@ -10,6 +10,7 @@ PIP_TESTS="\
 	fimmel.pip \
 	max.pip \
 	negative.pip \
+	seghir-vd.pip \
 	small.pip \
 	sor1d.pip \
 	square.pip \
@@ -18,5 +19,6 @@ PIP_TESTS="\
 
 for i in $PIP_TESTS; do
 	echo $i;
-	./isl_pip$EXEEXT -T < $srcdir/test_inputs/$i || exit
+	./isl_pip$EXEEXT --context=gbr -T < $srcdir/test_inputs/$i || exit
+	./isl_pip$EXEEXT --context=lexmin -T < $srcdir/test_inputs/$i || exit
 done
